import java.awt.Color;
import java.awt.Frame;
import java.util.ArrayList;
import javax.swing.JDialog;
import linearalgebra.LinearSystem;
import linearalgebra.MatrixInterface;
import linearalgebra.PermutationMatrix;
import linearalgebra.Util;
import linearalgebra.exception.OnlyZeroRowsError;
import linearalgebra.exception.SwapRowsException;
import linearalgebra.exception.UninvertibleMatrixException;
import linearalgebra.graph.GraphLine;
import linearalgebra.graph.GraphObject;
import linearalgebra.graph.GraphPanel;
import linearalgebra.graph.GraphVector;
import linearalgebra.graph.Viewport;
import linearalgebra.numeric.Matrix;
import linearalgebra.numeric.NumericColumn;
import linearalgebra.numeric.NumericMatrixInterface;
import linearalgebra.numeric.NumericRow;
import linearalgebra.numeric.NumericVector;

public class ExampleSet1 {
	public void example1() {
		Matrix a = new Matrix( new double[][] 
			{	{  2, 1, 1 },
				{  4, 1, 0 },
				{ -2, 2, 1 } } );
		NumericColumn y = new NumericColumn( new double[] { 1, -2, 7 } );
		LinearSystem s = new LinearSystem( a, y );
		s.combineRows(0, 1, -2.0);
		s.combineRows(0, 2,  1.0);
		System.out.println(s.toString());
		s.combineRows(1, 2,  3.0);
		System.out.println(s.toString());
		s.combineRows(2, 1,  -0.5);
		s.combineRows(2, 0,  0.25);
		System.out.println(s.toString());
		s.combineRows(1, 0,  1.0);
		System.out.println(s.toString());
		s.multiplyRow(0, 0.5);
		s.multiplyRow(1, -1.0);
		s.multiplyRow(2, -0.25);
		s.backSubstituteLower();
		System.out.println(s.toString());
		for( int i = 0; i < s.getVariableCount(); i++ ) {
			System.out.println(s.getSolution(i));
		}
		assert( s.getSolution(0).equals("a = -1.0") );
		assert( s.getSolution(1).equals("b = 2.0") );
		assert( s.getSolution(2).equals("c = 1.0") );
	}
	
	public void exercise1_2_1() {
		try {
			Matrix a = new Matrix( new double[][] 
				{	{  2, -3,  0 },
					{  4, -5,  1 },
					{  2, -1, -3 } } );
			NumericColumn y = new NumericColumn( new double[] { 3, 7, 5 } );
			LinearSystem s = new LinearSystem( a, y );
			// assume no zero pivots
			// eliminate
			for( int i=0; i < s.getEquationCount(); i++ ) {
				for( int j=i+1; j < s.getEquationCount(); j++ ) {
					double pivot = s.getPivot(i, j);
					s.combineRows(i, j, pivot);
					System.out.println(s.toString());
					System.out.println();
				}
			}
			// back substitute
			for( int i=s.getEquationCount()-1; i > 0; i-- ){
				for( int j=i-1; j >= 0; j-- ) {
					double pivot = s.getPivot(i, j);
					s.combineRows(i, j, pivot);
					System.out.println(s.toString());
					System.out.println();
				}
			}
			// normalize
			for( int i=0; i < s.getEquationCount(); i++ ) {
				s.normalizeRow(i);
			}
			s.backSubstituteLower();
			// print solutions
			for( int i = 0; i < s.getVariableCount(); i++ ) {
				System.out.println(s.getSolution(i));
			}
			assert( s.getSolution(0).equals("a = 3.0") );
			assert( s.getSolution(1).equals("b = 1.0") );
			assert( s.getSolution(2).equals("c = 0.0") );
		} catch( OnlyZeroRowsError e ) {
			System.out.println("Remaining rows are zero.");
		} catch( SwapRowsException e ) {
			System.out.println("System requires row swaps to solve.");
		}
	}	
	
	public void exercise1_2_2() {
		Matrix a = new Matrix( new double[][] 
			{	{  2, -1,  0,  0 },
				{ -1,  2, -1,  0 },
				{  0, -1,  2, -1 },
				{  0,  0, -1,  2 } } );
		NumericColumn y = new NumericColumn( new double[] { 0, 0, 0, 5 } );
		LinearSystem s = new LinearSystem( a, y );
		s.leftEliminate();
		System.out.println(s.toString());
		System.out.println();
		s.rightEliminate();
		System.out.println(s.toString());
		System.out.println();
		try {
			s.normalize();
		} catch( SwapRowsException e ) {
		} catch( OnlyZeroRowsError e ) {
		}
		System.out.println(s.toString());
		System.out.println();
		s.backSubstituteLower();
		// print solutions
		for( int i = 0; i < s.getVariableCount(); i++ ) {
			System.out.println(s.getSolution(i));
		}
		assert( s.getSolution(0).equals("a = 0.9999999999999998") );
		assert( s.getSolution(1).equals("b = 1.9999999999999996") );
		assert( s.getSolution(2).equals("c = 2.9999999999999996") );
	}
	
	public void exercise1_2_3() {
		Matrix a = new Matrix( new double[][] 
			{	{  1,  1,  1 },
				{  3,  3, -1 },
				{  1, -1,  1 } } );
		NumericColumn y = new NumericColumn( new double[] { -2, 6, -1} );
		LinearSystem s = new LinearSystem( a, y );
		s.leftEliminate();
		System.out.println(s.toString());
		System.out.println();
		s.backSubstituteUpper();
		System.out.println(s.toString());
		System.out.println();
		// print solutions
		for( int i = 0; i < s.getVariableCount(); i++ ) {
			System.out.println(s.getUnknownValue(i));
		}
		assert( s.getUnknownValue(0) == 1.5 );
		assert( s.getUnknownValue(1) == -0.5 );
		assert( s.getUnknownValue(2) == -3.0 );
	}
	
	public void exercise1_2_4() {
		final Viewport vp = new Viewport();
		final ArrayList<GraphLine> lines = new ArrayList<GraphLine>();
		vp.setViewport(-5, -5, 10, 10);
		
		Matrix a = new Matrix( new double[][] 
			{	{  1, -1 },
				{  3,  6 } } );
		NumericColumn y = new NumericColumn( new double[] {  0, 18 } );
		LinearSystem s = new LinearSystem( a, y );
		lines.add( new GraphLine( y.getRowValue(0)/a.getElement(0, 0), y.getRowValue(0)/a.getElement(0, 1), a.getElement(0, 0)/a.getElement(0, 1), Color.black));
		lines.add( new GraphLine( y.getRowValue(1)/a.getElement(1, 0), y.getRowValue(1)/a.getElement(1, 1), a.getElement(1, 0)/a.getElement(1, 1), Color.black));
		s.solve();
		lines.add( new GraphLine( y.getRowValue(1)/a.getElement(1, 0), y.getRowValue(1)/a.getElement(1, 1), a.getElement(1, 0)/a.getElement(1, 1), Color.red));
		for( int i = 0; i < s.getVariableCount(); i++ ) {
			System.out.println(s.getUnknownValue(i));
		}
		assert( s.getUnknownValue(0) == 2 );
		assert( s.getUnknownValue(1) == 2 );
		JDialog f = new JDialog((Frame)null, "Example 1.2.4", true);
		GraphObject[] lineArray = new GraphObject[lines.size()];
		lines.toArray(lineArray);
		f.getContentPane().add(new GraphPanel(lineArray, -15, -15, 15, 15));
		f.setSize(300,300);
		f.setVisible(true);
		f.dispose();
	}
	
	public void exercise1_2_7() {
		Matrix a = new Matrix( new double[][] 
			{	{  1, 1,  1 },
				{  1, 2,  2 },
				{  2, 3, -4 } } );
		NumericColumn y = new NumericColumn( new double[] {  6, 11, 3 } );
		LinearSystem s = new LinearSystem( a, y );
		s.solve();
		for( int i = 0; i < s.getVariableCount(); i++ ) {
			System.out.println(s.getUnknownValue(i));
		}
		assert( s.getUnknownValue(0) == 1 );
		assert( s.getUnknownValue(1) == 3 );
		assert( s.getUnknownValue(2) == 2 );
	}
	
	public void exercise1_3_1() {
		{
			Matrix a = new Matrix( new double[][] 
				{	{  4, 0,  1 },
					{  0, 1,  0 },
					{  4, 0,  1 } } );
			NumericColumn x = new NumericColumn( new double[] { 3, 4, 5 } );
			NumericColumn ax_soln = new NumericColumn( new double[] { 17, 4, 17 }); 
			MatrixInterface ax = a.product(x);
			System.out.println(ax.toString());
			assert( ax.equals(ax_soln) );
		}
		{
			Matrix a = new Matrix( new double[][] 
				{	{  1, 0,  0 },
					{  0, 1,  0 },
					{  0, 0,  1 } } );
			NumericColumn x = new NumericColumn( new double[] { 5, -2, 3 } );
			NumericColumn ax_soln = new NumericColumn( new double[] { 5, -2, 3 }); 
			MatrixInterface ax = a.product(x);
			System.out.println(ax.toString());
			assert( ax.equals(ax_soln) );
		}
		{
			Matrix a = new Matrix( new double[][] 
				{	{  2, 0 },
					{  1, 3 } } );
			NumericColumn x = new NumericColumn( new double[] { 1, 1 } );
			NumericColumn ax_soln = new NumericColumn( new double[] { 2, 4 }); 
			MatrixInterface ax = a.product(x);
			System.out.println(ax.toString());
			assert( ax.equals(ax_soln) );
			JDialog f = new JDialog((Frame)null, "Example 1.3.1", true);
			GraphObject[] lineArray = new GraphObject[5];
			lineArray[0] = new GraphVector( a.getNumericColumnView(0).getNumericVectorView(), new NumericVector(2), Color.black );
			lineArray[1] = new GraphVector( a.getNumericColumnView(1).getNumericVectorView(), new NumericVector(2), Color.red );
			lineArray[2] = new GraphVector( a.getNumericColumnView(0).getNumericVectorView(), a.getNumericColumnView(1).getNumericVectorView(), Color.black );
			lineArray[3] = new GraphVector( a.getNumericColumnView(1).getNumericVectorView(), a.getNumericColumnView(0).getNumericVectorView(), Color.red );
			lineArray[4] = new GraphVector( a.getNumericColumnView(0).plus(a.getNumericColumnView(1)).getNumericColumnView(0).getNumericVectorView(), new NumericVector(2), Color.blue );
			f.getContentPane().add(new GraphPanel(lineArray, -1, -1, 5, 5));
			f.setSize(300,300);
			f.setVisible(true);
			f.dispose();
		}
	}
	
	public void exercise1_3_2() {
		{
			Matrix a = new Matrix( new double[][] 
				{	{  4, 1 },
					{  5, 1 },
					{  6, 1 } } );
			NumericColumn x = new NumericColumn( new double[] { 1, 3 } );
			NumericColumn ax_soln = new NumericColumn( new double[] { 7, 8, 9 }); 
			NumericColumn ax = new NumericColumn(3);
			for( int i = 0; i < a.getColumnCount(); i++ ) {
				ax.plusEquals(a.getNumericColumnView(i).multiply(x.getRowValue(i)));
			}
			System.out.println(ax.toString());
			assert( ax.equals(ax_soln) );
		}
		{
			Matrix a = new Matrix( new double[][] 
				{	{  1, 2, 3 },
					{  4, 5, 6 },
					{  7, 8, 9 } } );
			NumericColumn x = new NumericColumn( new double[] { 0, 1, 0 } );
			NumericColumn ax_soln = new NumericColumn( new double[] { 2, 5, 8 }); 
			NumericColumn ax = new NumericColumn(3);
			for( int i = 0; i < a.getColumnCount(); i++ ) {
				ax.plusEquals(a.getNumericColumnView(i).multiply(x.getRowValue(i)) );
			}
			System.out.println(ax.toString());
			assert( ax.equals(ax_soln) );
		}
	}
	
	public void exercise1_3_3() {
		{
			NumericRow a = new NumericRow( new double[] { 1, -2, 7 } );
			NumericColumn x = new NumericColumn( new double[] { 1, -2, 7 } );
			NumericMatrixInterface ax = a.product(x);
			assert( ax.getColumnCount() == 1 );
			assert( ax.getRowCount() == 1 );
			assert( ax.getElement(0, 0) == 54 );
			double product = a.getNumericVectorView().innerProduct(x.getNumericVectorView());
			assert( product == 54 );			
		}
		{
			Matrix a = new Matrix( new double[][] {{ 1, -2, 7 }} );
			NumericColumn x = new NumericColumn( new double[] { 3, 5, 1 } );
			NumericMatrixInterface ax = a.product(x);
			assert( ax.getColumnCount() == 1 );
			assert( ax.getRowCount() == 1 );
			assert( ax.getElement(0, 0) == 0 );
			double product = a.getNumericRowView(0).getNumericVectorView().innerProduct(x.getNumericVectorView());
			assert( product == 0 );			
		}
	}
	
	public void exercise1_3_5() {
		Matrix a = new Matrix( new double[][] 
			{	{  3, -6,  0 },
				{  0,  2, -2 },
				{  1, -1, -1 } } );
		NumericColumn x = new NumericColumn( new double[] { 2, 1, 1 } );
		NumericColumn y = new NumericColumn( new double[] { 0, 0, 0 } );
		NumericColumn ax_soln = new NumericColumn( new double[] { 0, 0, 0 } );
		MatrixInterface ax = a.product(x);
		assert( ax.equals(ax_soln) );
		
		LinearSystem s = new LinearSystem(a, y);
		s.solve();
		for( int i = 0; i < s.getVariableCount(); i++ ) {
			System.out.println(s.getUnknownDescription(i) );
		}
		assert( s.getUnknownDescription(0).equals("a = 2.0*b") );
		assert( s.getUnknownDescription(1).equals("b = c") );
		assert( s.getUnknownDescription(2).equals("c is unbound.") );
	}
	
	public void exercise1_3_6() {
		Matrix a = new Matrix(3, 2);
		Matrix b = new Matrix(3, 2);
		for( int row = 0; row < 3; row++ ) {
			for( int col = 0; col < 2; col++ ) {
				a.setElement(row, col, row+col);
				b.setElement(row, col, Math.pow(-1, row+col));
			}
		}
		System.out.println( a.toString() );
		System.out.println( b.toString() );
	}
	
	public void exercise1_3_10() {
		Matrix a = new Matrix( new double[][] 
			{
				{ 1, 0, 0},
				{-2, 1, 0},
				{ 1, 0, 1},
			});
		Matrix b = new Matrix( new double[][] 
			{
				{ 2, 1, 0},
				{ 4, 1, 0},
				{-2, 2, 1},
			});
		NumericMatrixInterface c = a.product(b);
		Matrix soln = new Matrix( new double[][]
			{
				{ 2, 1, 0},
				{ 0,-1, 0},
				{ 0, 3, 1},
			});
		System.out.println(c.toString());
		assert( c.equals(soln) );
	}
	
	public void exercise1_3_11() {
		Matrix a = new Matrix( new double[][] 
			{
				{ 1, 0, 0},
				{-2, 1, 0},
				{-5, 3, 1},
			});
		Matrix b = new Matrix( new double[][] 
			{
				{ 2, 1, 0},
				{ 4, 1, 0},
				{-2, 2, 1},
			});
		NumericMatrixInterface c = a.product(b);
		Matrix soln = new Matrix( new double[][]
			{
				{ 2, 1, 0},
				{ 0,-1, 0},
				{ 0, 0, 1},
			});
		System.out.println(c.toString());
		assert( c.equals(soln) );
	}
	
	public void exercise1_3_12() {
		Matrix a = new Matrix( new double[][] 
			{
				{ 1, -1},
			});
		Matrix b = new Matrix( new double[][] 
			{
				{ 2 },
				{ 1 },
			});
		NumericMatrixInterface x = a.product(b);
		NumericMatrixInterface y = b.product(a);
		
		Matrix soln_x = new Matrix( new double[][]
			{
				{ 1 },
			});
		
		Matrix soln_y = new Matrix( new double[][]
			{
				{ 2, -2 },
				{ 1, -1 },
			});
		System.out.println(x.toString());
		System.out.println(y.toString());
		assert( x.equals(soln_x) );
		assert( y.equals(soln_y) );
	}
	
	public void exercise1_3_13() {
		Matrix a = new Matrix( new double[][] 
			{
				{ 8,-3},
				{-5, 2},
				{ 1, 0},
			});
		Matrix b = new Matrix( new double[][] 
			{
				{ 2},
				{ 1},
			});
		NumericMatrixInterface c = a.product(b);
		Matrix soln = new Matrix( new double[][]
			{
				{ 13},
				{ -8},
				{ 2},
			});
		System.out.println(c.toString());
		assert( c.equals(soln) );
	}
	
	public void exercise1_4_2() {
		double[][] va = {	{ 2, 1},
							{ 8, 7}};
		double[][] vb = {	{ 1, 0},
							{ 8, 1}};
		Matrix a = new Matrix( va );
		Matrix b = new Matrix( vb );
		PermutationMatrix p = new PermutationMatrix(2);
		NumericMatrixInterface L_a = (NumericMatrixInterface)a.factorL(p, false);
		NumericMatrixInterface L_b = (NumericMatrixInterface)b.factorL(p, false);
		Matrix solnA = new Matrix( new double[][] {
			{1, 0},
			{4, 1},
		});
		Matrix solnB = new Matrix( new double[][] {
			{1, 0},
			{8, 1},
		});
		assert( L_a.equals(solnA) );
		assert( L_b.equals(solnB) );
		assert( L_a.product(a).equals(new Matrix( va )) );
		assert( L_b.product(b).equals(new Matrix( vb )) );
	}
	
	public void exercise1_4_3() {
		double[][] va = {	{ 2, 3, 3 },
							{ 0, 5, 7 },
							{ 6, 9, 8 }};
		double[] vb = 	{ 2, 2, 5 };
		LinearSystem system = new LinearSystem(new Matrix(va), new NumericColumn(vb), false);
		system.solve();
		String str = system.toString();
		System.out.println( str );
		String soln = "2.0\t3.0\t3.0\t\t\ta\t\t2.0\n0.0\t5.0\t7.0\t\t\tb\t\t2.0\n0.0\t0.0\t-1.0\t\t\tc\t\t-1.0\n";
		assert( soln.equals( str ) );
	}
		
	public void exercise1_4_4() {
		try {
			double[][] va = {	{ 1, 0},
								{ 6, 1}};
			double[][] vb = {	{ 3, 0},
								{ 0, 2}};
			Matrix a = new Matrix( va );
			Matrix b = new Matrix( vb );
			Matrix i = new Matrix( 2 );
			NumericMatrixInterface ia = (NumericMatrixInterface)a.factorInverse();
			NumericMatrixInterface ib = (NumericMatrixInterface)b.factorInverse();
			a = new Matrix( va );
			b = new Matrix( vb );
			Matrix solnA = new Matrix( new double[][] {
				{1, 0},
				{-6, 1},
			});
			Matrix solnB = new Matrix( new double[][] {
				{1.0/3.0, 0},
				{0, 0.5},
			});
			assert( ia.equals(solnA) );
			assert( ib.equals(solnB) );
			assert( ia.product(a).equals(i) );
			assert( ib.product(b).equals(i) );
		} catch( UninvertibleMatrixException e ) {
			assert(false);
		}
	}
	
	public void exercise1_4_5() {
		double[][] va = {	{ 1, 0},
							{ 6, 1}};
		NumericMatrixInterface a = new Matrix( va );
		a = a.power(3);
		Matrix soln = new Matrix( new double[][] {
			{  1, 0},
			{ 18, 1}		
		});
		assert( a.equals(soln) );
	}
	
	public void exercise1_4_7() {
		double[][] va = {
			{ 2,  4,  0,  2},
			{ 0,  3,  3,  1},
			{ 2,  7,  9,  7},
			{ 0,  0,  6,  5},
		};
		double[] vb = {
			6, 4, 8, -4
		};
		NumericMatrixInterface a = new Matrix(va);
		NumericColumn b = new NumericColumn( vb );
		LinearSystem system = new LinearSystem( a, b );
		system.solve();
		String str = system.toString();
		System.out.println(str);
		String soln = "2.0\t4.0\t0.0\t2.0\t\t\ta\t\t6.0\n0.0\t3.0\t3.0\t1.0\t\t\tb\t\t4.0\n0.0\t0.0\t6.0\t4.0\t\t\tc\t\t-2.0\n0.0\t0.0\t0.0\t1.0\t\t\td\t\t-2.0\n";
		assert( soln.equals(str) );		
	}
	
	public void exercise1_4_9() {
		double[][] va = {
			{ 2, -1,  0},
			{-1,  2, -1},
			{ 0, -1,  2},
		};
		double[] vb = {
			0, 0, -6
		};
		NumericMatrixInterface a = new Matrix(va);
		NumericColumn b = new NumericColumn( vb );
		
		NumericMatrixInterface d = new Matrix(va);
		PermutationMatrix p = new PermutationMatrix(3);
		NumericMatrixInterface l = (NumericMatrixInterface)d.factorL(p);
		NumericMatrixInterface u = (NumericMatrixInterface)d.factorU(p);
		System.out.println(l);
		System.out.println(d);
		System.out.println(u);
		assert( d.isUpperRightTriangular() );
		assert( d.isLowerLeftTriangular() );
		assert( l.isLowerLeftTriangular() );
		assert( u.isUpperRightTriangular() );
		
		LinearSystem system = new LinearSystem( a, b );
		system.solve();
		String str = system.toString();
		System.out.println( str );
		
		for( int i = 0; i < system.getVariableCount(); i++ ) {
			System.out.println(system.getUnknownValue(i));
		}
		Util.setTolerance(0.00000000000001);
		assert( Util.isEqual(system.getUnknownValue(0), -1.5) );
		assert( Util.isEqual(system.getUnknownValue(1), -3.0) );
		assert( Util.isEqual(system.getUnknownValue(2), -4.5) );
		Util.setTolerance(0.0);
	}
	
	public void exercise1_5_1() {
		double[][] va = {
			{ 1,  4,  2},
			{-2, -8,  3},
			{ 0,  1,  1},
		};
		double[] vb = {
			-2, 32, 1
		};
		NumericMatrixInterface a = new Matrix(va);
		NumericColumn b = new NumericColumn( vb );
		LinearSystem system = new LinearSystem( a, b, false );
		system.solve();
		String str = system.toString();
		System.out.println(str);
		String soln = "1.0\t4.0\t2.0\t\t\ta\t\t-2.0\n0.0\t1.0\t1.0\t\t\tb\t\t1.0\n0.0\t0.0\t7.0\t\t\tc\t\t28.0\n";
		for( int i = 0; i < system.getVariableCount(); i++ ) {
			System.out.println(system.getUnknownValue(i));
		}
		assert( system.getUnknownValue(0) == 2.0);
		assert( system.getUnknownValue(1) == -3.0);
		assert( system.getUnknownValue(2) == 4.0);
		assert( soln.equals(str) );		
	}	
	
	public void exercise1_5_2() {
		double[][] va1 = {
			{ 0,  1},
			{ 2,  3},
		};
		double[][] va2 = {
			{ 1,  2,  3},
			{ 2,  4,  2},
			{ 1,  1,  1},
		};
		
		NumericMatrixInterface a1 = new Matrix(va1);
		PermutationMatrix p1 = new PermutationMatrix(2);
		MatrixInterface l1 = a1.factorL(p1);
		MatrixInterface u1 = a1.factorU(null);
		System.out.println(p1);
		System.out.println(l1);
		System.out.println(a1);
		System.out.println(u1);
		
		NumericMatrixInterface a2 = new Matrix(va2);
		PermutationMatrix p2 = new PermutationMatrix(3);
		MatrixInterface l2 = a2.factorL(p2);
		MatrixInterface u2 = a2.factorU(null);
		System.out.println(p2);
		System.out.println(l2);
		System.out.println(a2);
		System.out.println(u2);
		
	}
	
	public void exercise1_5_6() {
		double[][] va = {
			{ 0,  1, -1},
			{ 1, -1,  0},
			{ 1,  0, -1},
		};
		double[] vb1 = {2, 2, 2};
		double[] vb2 = {0, 0, 0};
		
		LinearSystem s1 = new LinearSystem( new Matrix(va), new NumericColumn(vb1) );
		s1.solve();
		System.out.println(s1);
		for( int i = 0; i < s1.getVariableCount(); i++ ) {
			System.out.println(s1.getSolution(i));
		}
		System.out.println( s1.getSolvableDescription() );
		assert( s1.isInconsistent() );
		
		System.out.println();
		
		LinearSystem s2 = new LinearSystem( new Matrix(va), new NumericColumn(vb2) );
		s2.solve();
		System.out.println(s2);
		for( int i = 0; i < s2.getVariableCount(); i++ ) {
			System.out.println(s2.getSolution(i));
		}
		System.out.println( s2.getSolvableDescription() );
		assert( s2.isUnderdetermined() );
	}
	
	public void exercise1_5_10() {
		try {
			double[][] va = {
				{  1,  1,  1},
				{  0,  1,  1},
				{  0,  0,  1},
			};
			Matrix a = new Matrix( va );
			System.out.println(a);
			Matrix ia = (Matrix)a.factorInverse();
			System.out.println(ia);
			System.out.println(a);
			a = new Matrix( va );
			MatrixInterface i = a.product(ia);
			System.out.println(i);
			assert( i.isIdentity() );
		} catch( UninvertibleMatrixException e ) {
			assert(false);
		}
	}
	
	public void exercise1_5_11() {
		try {
			double[][] va = {
				{  1,  1},
				{  3,  3},
			};
			Matrix a = new Matrix( va );
			Matrix ia = (Matrix)a.factorInverse();
			assert(false);
		} catch( UninvertibleMatrixException e ) {
		}
	}
	
	public void exercise1_5_12() {
		try {
			double[][] va = {
				{  2, -1,  1},
				{ -1,  2, -1},
				{  0, -1,  2},
			};
			Matrix a = new Matrix( va );
			System.out.println(a);
			Matrix ia = (Matrix)a.factorInverse();
			System.out.println(ia);
			System.out.println(a);
			a = new Matrix( va );
			MatrixInterface i = a.product(ia);
			System.out.println(i);
			Util.setTolerance(0.0000000000000001);
			assert( i.isIdentity() );
			Util.setTolerance(0.0);
		} catch( UninvertibleMatrixException e ) {
			assert(false);
		}
	}
	
	public void exercise1_5_13() {
		try {
			double[][] va = {
				{  2,  1,  4,  6},
				{  0,  3,  8,  5},
				{  0,  0,  0,  7},
				{  0,  0,  0,  9},
			};
			Matrix a = new Matrix( va );
			Matrix ia = (Matrix)a.factorInverse();
			assert(false);
		} catch( UninvertibleMatrixException e ) {
		}
	}
	
	public void exercise1_5_15() {
		double[][] va = {
			{3},
			{1}
		};
		double[][] vb = {
			{2},
			{2}
		};
		Matrix a = new Matrix( va );
		Matrix b = new Matrix( vb );
		MatrixInterface atb = ((NumericMatrixInterface)a.getTranspose()).product(b);
		MatrixInterface bta = ((NumericMatrixInterface)b.getTranspose()).product(a);
		MatrixInterface abt = a.product(((NumericMatrixInterface)b.getTranspose()));
		MatrixInterface bat = b.product(((NumericMatrixInterface)a.getTranspose()));
		System.out.println( atb );
		System.out.println( bta );
		System.out.println( abt );
		System.out.println( bat );
		assert( atb.equals( bta.getTranspose() ) );
		assert( abt.equals( bat.getTranspose() ) );
		assert( atb.toString().equals( "8.0\t\n" ) );
		assert( abt.toString().equals( "6.0\t6.0\t\n2.0\t2.0\t\n" ) );
	}

	public void exercise1_5_20() {
		double[][] va = {
			{  1,  3,  5},
			{  3, 12, 18},
			{  5, 18, 30},
		};
		Matrix a = new Matrix(va);
		assert( a.isPositiveDefinite() );
		Matrix L = (Matrix)a.factorL(null, false);
		System.out.println(L);
		System.out.println(a);
		for( int i = 0; i < a.getRowCount(); i++ ) {
			double p = a.getElement(i, i);
			p = Math.sqrt(p);
			L.getNumericColumnView(i).multiplyEquals(p);
		}
		for( int row = 0; row < L.getRowCount(); row++ ) {
			for( int col = 0; col <= row; col++ ) {
				a.setElement(col, row, L.getElement(row, col));
			}
		}
		System.out.println(L);
		System.out.println(a);
		assert( a.equals( L.getTranspose() ) );
		MatrixInterface a2 = L.product( a );
		System.out.println(a2);
		assert( a2.equals( new Matrix(va) ) );	
	}
	
	public void exercise1_6_5() {
		double[][] va = {
			{  1.0/1.0, 1.0/2.0, 1.0/3.0},
			{  1.0/2.0, 1.0/3.0, 1.0/4.0},
			{  1.0/3.0, 1.0/4.0, 1.0/5.0},
		};
		Matrix a = new Matrix(va);
		try {
			Matrix ia = (Matrix)a.factorInverse();
			a = new Matrix(va);
			System.out.println(ia);
			MatrixInterface i = a.product(ia);
			System.out.println(i);
			Util.setTolerance(0.00000000000001);
			assert( i.isIdentity() );			
		} catch( UninvertibleMatrixException e) {
			assert(false);
		}
	}
	
	public void exercise1_6_6() {
		double[][] va = {
			{  0.001, 0.000},
			{  1.000, 1000 },
		};
		Matrix a = new Matrix(va);
		a.factorL(new PermutationMatrix(2), false);
		System.out.println("Without partial pivots:");
		System.out.println(a);
		
		a = new Matrix(va);
		a.factorL(new PermutationMatrix(2));
		System.out.println("With partial pivots:");
		System.out.println(a);
	}
	
	public static void main( String[] args ) {		
		ExampleSet1 ex = new ExampleSet1();		
		Util.setTolerance(0.0);
		
//		ex.example1();
//		ex.exercise1_2_1();
//		ex.exercise1_2_2();
//		ex.exercise1_2_3();
//		ex.exercise1_2_4();
//		ex.exercise1_2_7();
		ex.exercise1_3_1();
//		ex.exercise1_3_2();
//		ex.exercise1_3_3();
//		ex.exercise1_3_5();
//		ex.exercise1_3_6();
//		ex.exercise1_3_10();
//		ex.exercise1_3_11();
//		ex.exercise1_3_12();
//		ex.exercise1_3_13();
//		ex.exercise1_4_2();
//		ex.exercise1_4_3();
//		ex.exercise1_4_4();
//		ex.exercise1_4_5();
//		ex.exercise1_4_7();
//		ex.exercise1_4_9();
//		ex.exercise1_5_1();
//		ex.exercise1_5_2();
//		ex.exercise1_5_6();
//		ex.exercise1_5_10();
//		ex.exercise1_5_11();
//		ex.exercise1_5_12();
//		ex.exercise1_5_13();
//		ex.exercise1_5_15();
//		ex.exercise1_5_20();
//		ex.exercise1_6_5();

		// Do section 1.6 with matrix approximations of DiffEq
//		ex.exercise1_6_6();
		System.exit(0);
	}
}


