package linearalgebra;

import linearalgebra.numeric.*;

public class GeneralSolution {
	private NumericMatrixInterface a;
	private UnknownValue[] x;
	private NumericColumnInterface c;
	
	public GeneralSolution( NumericMatrixInterface a, UnknownValue[] x, NumericColumnInterface c ) {
		this.a = a;
		this.x = x;
		if( c.getNumericVectorView().isZero() ) {
			c = null;
		}
		this.c = c;
	}
	
	public NumericMatrixInterface getMatrixView() {
		return a;
	}
	
	private static final String additionSeparator = "\t+\t";
	
	public String toString() {
		StringBuffer rc = new StringBuffer();
		int maxVarLength = getMaxVariableNameLength();
		int additionRow = a.getRowCount()/2;
		for( int row = 0; row < a.getRowCount(); row++ ) {
			for( int col = 0; col < a.getColumnCount(); col++ ) {
				if( row == additionRow ) {
					if( col > 0 ) {
						rc.append(additionSeparator);
					}
					// print variable
					rc.append(x[col].getName());
					rc.append("*");
				} else {
					// print whitespace
					if( col > 0 ) {
						rc.append("\t \t");
					}
					rc.append( padStringLeft("", maxVarLength+1) );				
				}
				rc.append( a.getElement(row, col) );
			}
			if( c != null ) {
				if( a.getColumnCount() > 0 ) {
					if( row == additionRow ) {
						rc.append(additionSeparator);
					} else {
						rc.append("\t \t");
					}
				}
				rc.append(c.getRowValue(row));
			} else if( a.getColumnCount() == 0 ) {
				rc.append("0.0");
			}
			rc.append("\n");
		}
		return rc.toString();
	}
	
	private final int getMaxVariableNameLength() {
		int maxVarLength = 0;
		for( int i = 0; i < x.length; i++ ) {
			maxVarLength = Math.max( maxVarLength, x[i].getName().length() );
		}
		return maxVarLength;
	}
	
	private final static String padStringLeft( String str, int size ) {
		StringBuffer rc = new StringBuffer();
		for( int i = 0; i < (size-str.length()); i++ ) {
			rc.append(' ');
		}
		rc.append(str);
		return rc.toString();
	}
}