package linearalgebra.symbolic;

public interface SymbolicValue {
	public static final int LEAF = 0;
	public static final int ADDS = 1;
	public static final int MULT = 2;

	public SymbolicValue scale( SymbolicValue v );
	public double getNumericValue() throws linearalgebra.exception.UnboundValueError;
	
	/**
	 * Checks if each child in the tree should be operated on first according
	 * to the operator precedence.
	 */
	// Tree operations
	
	public boolean isNormal();
	
	/**
	 * Returns an order-of-operations indicator. Lower numbers mean lower
	 * precedence. This does not influence the execution order of the tree.
	 * It was originally intended to allow the tree to insert parenthesis
	 * when printing itself.
	 */
	public int getPrecedence();
	
	/**
	 * Returns the number of child nodes beneath this tree.
	 */
	public int getChildCount();
	
	/**
	 * Returns the child node specifed by this index. 
	 * Throws ArrayOutOfBoundsException if index is invalid.
	 * 
	 * @param index
	 */
	public SymbolicValue getChild( int index );

	/**
	 * Returns the number of nodes that have this subtree as a child.
	 */
	public int getParentCount();
	
	/**
	 * Adds a parent to the nodes parent list.
	 * 
	 * @param parent
	 */
	public void addParent( SymbolicValue parent );
	public void removeParent( SymbolicValue parent );
	
	/**
	 * Returns the parent specified by the index.
	 * Throws an ArrayOutOfBoundsException if the index is invalid.
	 * 
	 * @param index
	 */
	public SymbolicValue getParent( int index );
	
	/**
	 * Searches the tree for the SymbolicValue v. Returns true if it contains
	 * the element, and false otherwise. The parent is passed in to allow
	 * easier tree traversal.
	 * 
	 * @param v
	 * @param parent
	 */
	public boolean contains( SymbolicValue v, SymbolicValue parent );
	
	
	/**
	 * Invalidates cached information about the tree.
	 */
	public void subTreeChanged();
}